/*
**  Copyright 2000 Compaq Computer Corporation
**  
**  COMPAQ Registered in U.S. Patent and Trademark Office.
**  
**  Confidential computer software.  Valid license from Compaq required for
**  possession, use or copying.  Consistent with FAR 12.211 and 12.212,
**  Commerical Computer Software, Computer Software Documentation, and
**  Technical Data for Commmerical Items are licensed to the U.S. Government
**  under vendor's standard commerical license.
*/

/*
**
** Example program to solve symmetric positive definite system of equations.
**
*/

#include <stdio.h>
#include <stdlib.h>
#include "cxml_dss.h"

/*
** Define the array and rhs vectors
*/

#define nRows 5
#define nCols 5
#define nNonZeros 9
#define nRhs  1

static _INTEGER_t          rowIndex[nRows+1]  = { 1, 6, 7, 8, 9, 10 };
static _INTEGER_t          columns[nNonZeros] = { 1, 2, 3, 4, 5, 2, 3, 4, 5
};
static _DOUBLE_PRECISION_t values[nNonZeros]  = { 9, 1.5, 6, .75, 3, 0.5,
12, .625, 16 };
static _DOUBLE_PRECISION_t rhs[nCols]         = { 1, 2, 3, 4, 5 };
void main() {
int i;

/* Allocate storage for the solver handle and the right-hand side.  */

_DOUBLE_PRECISION_t solValues[nRows];
_CXML_DSS_HANDLE_t  handle;
_INTEGER_t          error;

/* --------------------- */
/* Initialize the solver */
/* --------------------- */
error = dss_create(handle, CXML_DSS_DEFAULTS );
if ( error != CXML_DSS_SUCCESS ) goto printError;

/* ------------------------------------------- */
/* Define the non-zero structure of the matrix */
/* ------------------------------------------- */

error = dss_define_structure(
handle, CXML_DSS_SYMMETRIC, rowIndex, nRows, nCols,
columns, nNonZeros );
if ( error != CXML_DSS_SUCCESS ) goto printError;

/* ------------------ */
/* Reorder the matrix */
/* ------------------ */

error = dss_reorder( handle, CXML_DSS_DEFAULTS, 0);
if ( error != CXML_DSS_SUCCESS ) goto printError;

/* ------------------ */
/*  Factor the matrix */
/* ------------------ */

error = dss_factor_real( handle, CXML_DSS_POSITIVE_DEFINITE, values );
if ( error != CXML_DSS_SUCCESS ) goto printError;

/* ------------------------ */
/*  Get the solution vector */
/* ------------------------ */

error = dss_solve_real( handle, CXML_DSS_DEFAULTS, rhs, nRhs, solValues );
if ( error != CXML_DSS_SUCCESS ) goto printError;
/* -------------------------- */
/*  Deallocate solver storage */
/* -------------------------- */
error = dss_delete( handle, CXML_DSS_DEFAULTS );
if ( error != CXML_DSS_SUCCESS ) goto printError;
/* ---------------------- */
/*  Print solution vector */
/* ---------------------- */

printf(" Solution array: ");
for(i = 0; i< nCols; i++)
printf(" %g", solValues[i] );
printf("\n");
return;
printError:
printf("Solver returned error code %d\n", error);
}

